@@ -915,7 +915,7 @@ class Line(ModelSQL, ModelView):
                             else:
                                 line_amount -= tax_line['amount']
                             tax_amount += tax_line['amount'] * \
-                                tax_line['tax'][key + '_tax_sign']
+                                getattr(tax_line['tax'], key + '_tax_sign')
                     line_amount = line.account.company.currency.round(
                         line_amount)
                     tax_amount = line.account.company.currency.round(
@@ -1066,7 +1066,7 @@ class Line(ModelSQL, ModelView):
                     tax_line = TaxLine(**TaxLine.default_get(
                             TaxLine._fields.keys()))
 
-                    tax_line.amount = base_amounts[code_id, tax_id],
+                    tax_line.amount = base_amounts[code_id, tax_id]
                     tax_line.currency_digits = self.account.currency_digits
                     tax_line.code = code_id
                     tax_line.tax = tax_id
